from typing import List, Optional
from pydantic import BaseModel, Field
import requests
from ..config.config import ConfigProducteca
import logging

_logger = logging.getLogger(__name__)


class SalesOrderProduct(BaseModel):
    id: int
    name: str
    code: str
    brand: str


class SalesOrderVariationAttribute(BaseModel):
    key: str
    value: str


class SalesOrderVariation(BaseModel):
    id: int
    attributes: List[SalesOrderVariationAttribute]
    sku: str
    thumbnail: str


class SalesOrderLine(BaseModel):
    product: SalesOrderProduct
    variation: SalesOrderVariation
    quantity: int
    price: float


class SalesOrderCard(BaseModel):
    payment_network: str
    first_six_digits: int
    last_four_digits: int
    cardholder_identification_number: str
    cardholder_identification_type: str
    cardholder_name: str


class SalesOrderPaymentIntegration(BaseModel):
    integration_id: str
    app: int


class SalesOrderPayment(BaseModel):
    date: str
    amount: float
    coupon_amount: float
    status: str
    method: str
    integration: SalesOrderPaymentIntegration
    transaction_fee: float
    installments: int
    card: SalesOrderCard
    notes: str
    has_cancelable_status: bool
    id: int


class SalesOrderIntegration(BaseModel):
    alternate_id: str
    integration_id: int
    app: int


class SalesOrderShipmentProduct(BaseModel):
    product: int
    variation: int
    quantity: int


class SalesOrderShipmentMethod(BaseModel):
    tracking_number: str
    tracking_url: str
    courier: str
    mode: str
    cost: float
    type: str
    eta: str
    status: str


class SalesOrderShipmentIntegration(BaseModel):
    id: int
    integration_id: str
    app: int
    status: str


class SalesOrderShipment(BaseModel):
    date: str
    products: List[SalesOrderShipmentProduct]
    method: SalesOrderShipmentMethod
    integration: SalesOrderShipmentIntegration


class SalesOrderResultItem(BaseModel):
    codes: List[str]
    contact_id: int
    currency: str
    date: str
    delivery_method: str
    delivery_status: str
    id: str
    integration_ids: List[str]
    integrations: List[SalesOrderIntegration]
    invoice_integration_app: int
    invoice_integration_id: str
    lines: List[SalesOrderLine]
    payments: List[SalesOrderPayment]
    payment_status: str
    payment_term: str
    product_names: List[str]
    reserving_product_ids: str
    sales_channel: int
    shipments: List[SalesOrderShipment]
    tracking_number: str
    skus: List[str]
    status: str
    tags: List[str]
    warehouse: str
    company_id: int
    shipping_cost: float
    contact_phone: str
    brands: List[str]
    courier: str
    order_id: int
    updated_at: str
    invoice_integration_created_at: str
    invoice_integration_document_url: str
    has_document_url: bool
    integration_alternate_ids: str
    cart_id: str
    amount: float
    has_any_shipments: bool


class SearchSalesOrderResponse(BaseModel):
    count: int
    results: List[SalesOrderResultItem]


class SearchSalesOrderParams(BaseModel):
    top: Optional[int]
    skip: Optional[int]
    filter: Optional[str] = Field(default=None, alias="$filter")
    class Config:
        validate_by_name = True

class SearchSalesOrder:
    endpoint: str = "search/salesorders"


    @classmethod
    def search_saleorder(cls, config: ConfigProducteca, params: SearchSalesOrderParams):
        headers = config.headers
        url = config.get_endpoint(cls.endpoint)
        new_url = f"{url}?$filter={params.filter}&top={params.top}&skip={params.skip}"
        response = requests.get(
            new_url,
            headers=headers,
        )
        return response.json(), response.status_code





from typing import List, Optional
from pydantic import BaseModel, Field
import requests
from ..config.config import ConfigProducteca


class FacetValue(BaseModel):
    count: int
    value: str
    label: str


class Facet(BaseModel):
    key: str
    value: List[FacetValue]
    is_collection: bool
    translate: bool


class SearchStocks(BaseModel):
    warehouse: str
    quantity: int
    reserved: int


class SearchPrices(BaseModel):
    price_list_id: int
    price_list: str
    amount: float
    currency: str


class SearchIntegration(BaseModel):
    app: Optional[int]
    integration_id: Optional[str]
    permalink: Optional[str]
    status: Optional[str]
    listing_type: Optional[str]
    safety_stock: Optional[int]
    synchronize_stock: Optional[bool]
    is_active: Optional[bool]
    is_active_or_paused: Optional[bool]
    id: Optional[int]


class SearchDeals(BaseModel):
    campaign: str
    product: int
    variation: str
    deal_price: float
    discount: float
    regular_price: float
    enabled: bool
    currency: str
    id: str


class SearchResultItem(BaseModel):
    search_score: float = Field(..., alias='@search.score')
    id: int
    product_id: int
    company_id: int
    name: str
    code: str
    skus: List[str]
    brand: str
    category: str
    thumbnail: str
    stocks: List[SearchStocks]
    warehouses_with_stock: List[str]
    total_stock: int
    has_pictures: bool
    buying_price: float
    prices: List[SearchPrices]
    integration_ids: List[str]
    integration_apps: List[str]
    integrations: List[SearchIntegration]
    campaigns: List[str]
    app: Optional[int]
    status: Optional[str]
    synchronize_stock: Optional[bool]
    listing_type: Optional[str]
    price_amount: Optional[float]
    price_currency: Optional[str]
    category_id: Optional[str]
    category_base_id: Optional[str]
    category_l1: Optional[str]
    category_l2: Optional[str]
    category_l3: Optional[str]
    category_l4: Optional[str]
    category_l5: Optional[str]
    category_l6: Optional[str]
    has_category: Optional[bool]
    category_fixed: Optional[bool]
    accepts_mercadoenvios: Optional[bool]
    shipping_mode: Optional[str]
    local_pickup: Optional[bool]
    mandatory_free_shipping: Optional[bool]
    free_shipping: Optional[bool]
    free_shipping_cost: Optional[float]
    template: Optional[str]
    youtube_id: Optional[str]
    warranty: Optional[str]
    permalink: Optional[str]
    domain: Optional[str]
    attribute_completion_status: Optional[str]
    attribute_completion_count: Optional[int]
    attribute_completion_total: Optional[int]
    deals: Optional[SearchDeals]
    campaign_status: Optional[List[str]]
    size_chart: Optional[str]
    channel_status: Optional[List[str]]
    channel_category_l1: Optional[List[str]]
    channel_category_l2: Optional[List[str]]
    channel_category_l3: Optional[List[str]]
    channel_category_id: Optional[List[str]]
    channel_synchronizes_stock: Optional[List[str]]
    channel_has_category: Optional[List[str]]
    catalog_products_status: Optional[List[str]]
    metadata: Optional[List[str]]
    integration_tags: Optional[List[str]]
    variations_integration_ids: Optional[List[str]]
    channel_pictures_templates: Optional[List[str]]
    channel_pictures_templates_apps: Optional[List[str]]


class SearchProductResponse(BaseModel):
    count: int
    facets: List[Facet]
    results: List[SearchResultItem]


class SearchProductParams(BaseModel):
    top: Optional[int]
    skip: Optional[int]
    filter: Optional[str] = Field(default=None, alias='$filter')
    search: Optional[str]
    sales_channel: Optional[str] = Field(default='2', alias='salesChannel')


class SearchProduct:
    endpoint: str = 'search/products'

    @classmethod
    def search_product(cls, config: ConfigProducteca, params: SearchProductParams) -> SearchProductResponse:
        headers = config.headers
        url = config.get_endpoint(cls.endpoint)
        response = requests.get(url, headers=headers, params=params.dict(by_alias=True, exclude_none=True))
        return SearchProductResponse(**response.json())



from pydantic import BaseModel, PrivateAttr
from abc import ABC, abstractmethod


class APIConfig(BaseModel, ABC):
    base_url: str = 'https://api-external.producteca.com'

    @property
    @abstractmethod
    def headers(self) -> dict:
        pass

    def get_endpoint(self, endpoint: str) -> str:
        return f'{self.base_url}/{endpoint}'


class ConfigProducteca(APIConfig):
    token: str
    api_key: str

    @property
    def headers(self) -> dict:
        return {
            "Content-Type": "application/json",
            "authorization": f"Bearer {self.token}",
            "x-api-key": self.api_key,
            "Accept": "*/*"
        }



import unittest
from unittest.mock import patch, MagicMock
from .shipment import Shipment, ShipmentProduct, ShipmentMethod, ShipmentIntegration, ConfigProducteca

class TestShipment(unittest.TestCase):

    @patch('requests.post')
    def test_create_shipment(self, mock_post):
        # Arrange
        config = ConfigProducteca()
        sale_order_id = 123
        products = [ShipmentProduct(product=1, variation=2, quantity=3)]
        method = ShipmentMethod(trackingNumber="TN123", trackingUrl="http://track.url", courier="DHL", mode="air", cost=10.5, type="express", eta=5, status="shipped")
        integration = ShipmentIntegration(id=1, integrationId="int123", app=10, status="active")
        payload = Shipment(date="2023-01-01", products=products, method=method, integration=integration)

        mock_response = MagicMock()
        mock_response.status_code = 201
        mock_response.json.return_value = {'success': True}
        mock_post.return_value = mock_response

        # Act
        status_code, response_json = Shipment.create(config, sale_order_id, payload)

        # Assert
        self.assertEqual(status_code, 201)
        self.assertEqual(response_json, {'success': True})
        mock_post.assert_called_once()

    @patch('requests.put')
    def test_update_shipment(self, mock_put):
        # Arrange
        config = ConfigProducteca()
        sale_order_id = 123
        shipment_id = 'abc'
        products = [ShipmentProduct(product=4, quantity=7)]
        method = ShipmentMethod(courier="FedEx", cost=15.0)
        integration = ShipmentIntegration(status="pending")
        payload = Shipment(date="2023-02-02", products=products, method=method, integration=integration)

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = {'updated': True}
        mock_put.return_value = mock_response

        # Act
        status_code, response_json = Shipment.update(config, sale_order_id, shipment_id, payload)

        # Assert
        self.assertEqual(status_code, 200)
        self.assertEqual(response_json, {'updated': True})
        mock_put.assert_called_once()

if __name__ == '__main__':
    unittest.main()